import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;

/**
 * GetGlitschi
 * by D. Reichert
 * Version 1.0 on 27/02/2010
 * 
 * All graphics by D. Reichert
 */

public class Glitschi  extends SmoothMover
{   
    private static final int GLITSCHI_STANDARD_SCALE = 50;
    private static final int GLITSCHI_STANDARD_ROTATION = 270;

    private GreenfootImage glitschiGerade = new GreenfootImage("Glitschi1.png");
    private GreenfootImage glitschiRucksack = new GreenfootImage("GlitschiRucksack2.png");
    
    private static int lives = 3;
    private static int punkte = 0;
    private static boolean fischAufgenommen;
    private boolean glitschiScale = true;
    public static int wasserCounter = 200;

    // Standardkonstruktor fr Glitschi.    
    public Glitschi(int leben, int punkte) {
        this.lives = leben;
        this.punkte = punkte;
        getImage().scale(GLITSCHI_STANDARD_SCALE, GLITSCHI_STANDARD_SCALE);
        setRotation(GLITSCHI_STANDARD_ROTATION);
    }
    
    // Bestimmter Konstruktor fr Glitschi.    
    public Glitschi(int scaling, int rotation, int leben, int punkte) {
        this.lives = leben;
        this.punkte = punkte;
        getImage().scale(scaling, scaling);
        setRotation(rotation);
    }

     // Act-Methode.
    public void act() {
        move();
        checkKeys();
        checkSchnee();
        checkRandSchnee();
        checkOel();
        checkWasser();
        checkFisch();
        fischTragen();
        glitschiScaling();
        checkIgluBarriere();
        fischAbgeben();
        checkTod();
        setzeGlitschi();
        wasserCounter++;
    }  

    // Tastenabfragen.
    private void checkKeys() {     
        if (Greenfoot.isKeyDown("left")) {
            setRotation(getRotation() - 5);
        }
        if (Greenfoot.isKeyDown("right")) {
            setRotation(getRotation() + 5);
        }
        if (Greenfoot.isKeyDown("up")) {
           addForce(new Vector(getRotation(), 0.1));
        }
        if (Greenfoot.isKeyDown("down")) {
            addForce(new Vector(getRotation(), -0.05));
        }
        if (Greenfoot.isKeyDown("a")) {
           addForce(new Vector(getRotation()+270, 0.1));
        }
        if (Greenfoot.isKeyDown("d")) {
            addForce(new Vector(getRotation()+90, 0.1));
        }
    }    
    
    // Gibt die Leben zurueck.
    public static int getLives() {
        return lives;
    }
    
    // Gibt die Punkte zurueck.
    public static int getPunkte() {
        return punkte;
    }
    
    // Der Schnee verlangsamt die Bewegung von Glitschi. 
    // Wird der Schnee beruehrt und die eigene Geschwindigkeit ist
    // kleiner als 1, wird Glitschi verlangsamt.
    public void checkSchnee() {        
        if (checkSchneeKontakt() == true && getSpeed() > 0.2) {
            accelerate(0.9);
        }
    }
    
    // Gibt einen Rueckgabewert, ob ein Schnee-Objekt beruehrt wird.
    public boolean checkSchneeKontakt() {
        Actor schnee = (Schnee) getOneObjectAtOffset(30, 30, Schnee.class);       
        return schnee != null;
    }

    // Der Randschnee verlangsamt die Bewegung von Glitschi. 
    // Wird der Randschnee beruehrt und die eigene Geschwindigkeit ist
    // kleiner als 1, wird Glitschi verlangsamt.
    public void checkRandSchnee() {        
        if (checkRandSchneeKontakt() == true && getSpeed() > 0.2) {
            accelerate(0.9);
        }
    }
    
    // Gibt einen Rueckgabewert, ob ein Randschnee-Objekt beruehrt wird.
    public boolean checkRandSchneeKontakt() {
        Actor randschnee = (Randschnee) getOneIntersectingObject(Randschnee.class);       
        return randschnee != null;
    }
  
    // Das Oel setzt Glitschi wieder in sein Iglu und zieht ihm ein Leben ab.
    // setzt den SpeedVektor wieder auf 0.
    public void checkOel() {        
        Level level = (Level) getWorld();
        if (checkOelKontakt() == true) {            
            setLocation(level.GLITSCHI_START_X, level.GLITSCHI_START_Y);
            setRotation(level.GLITSCHI_ROTATION);
            stop();
            lives--;
            if (lives > 1) {
                Greenfoot.playSound("OelKontakt.wav");
            }
            if (lives == 1) {
                Greenfoot.playSound("EinLeben.wav");
            }
            if (fischAufgenommen) {   
                level.giveFisch();
                fischAufgenommen = false;   
            }
        }
    }
    
    // Gibt einen Rueckgabewert, ob ein Randschnee-Objekt beruehrt wird.
    public boolean checkOelKontakt() {
        Actor oel = (Oel) getOneObjectAtOffset(0, 0, Oel.class);       
        return oel != null;
    }
    
    // Das Wasser teleportiert Glitschi an eine beliebige Stelle
    public void checkWasser() {
        Wasser wasser = (Wasser) getOneObjectAtOffset(0, 0, Wasser.class);
        if (checkWasserKontakt() && wasserCounter >= 300) {
            wasser.teleportWasser();
            setLocation(wasser.wasserX(), wasser.wasserY());
            Greenfoot.playSound("WasserTp.wav");
            wasserCounter = 0;
            stop();
        }
    }
    
    // Gibt einen Rueckgabewert, ob das Wasser-Objekt beruehrt wird.
    public boolean checkWasserKontakt() {
        Actor wasser = (Wasser) getOneObjectAtOffset(0, 0, Wasser.class);
        return wasser != null;
    }

    // Fischfang ist das Ziel! Hier wird ueberprueft,  ob Glitschi einen Fisch hat
    public void checkFisch() {        
        Actor fisch = (Fisch) getOneObjectAtOffset(0, 0, Fisch.class);
        if (checkFischKontakt() == true) {            
            fischAufgenommen = true;
            getWorld().removeObject(fisch);
            Greenfoot.playSound("FischAufnehmen.wav");
        }
    }
    
    // Gibt einen Rueckgabewert, ob ein Fisch-Objekt beruehrt wird.
    public boolean checkFischKontakt() {
        Actor fisch = (Fisch) getOneObjectAtOffset(0, 0, Fisch.class);       
        return fisch != null;
    }    
    
    // Aendert das Bild, wenn ein Fisch aufgenommen wurde
    public void fischTragen() {
        if (fischAufgenommen && getImage() != glitschiRucksack) {
            setImage(glitschiRucksack);
            glitschiScale = false;
        }
        if (!fischAufgenommen && getImage() != glitschiGerade) {
            setImage(glitschiGerade);
            glitschiScale = false;
        }
    }
    
    // Glitschi nachskalieren, da er beim Bildwechsel immer zu gross war
    public void glitschiScaling() {
        if (!glitschiScale) {
            getImage().scale(GLITSCHI_STANDARD_SCALE, GLITSCHI_STANDARD_SCALE);
            glitschiScale = true;
        }
    }

    // Die unsichtbare Barriere am Iglu verhindert, dass man von allen Seiten ins Iglu kommt. 
    public void checkIgluBarriere() {        
        if (checkIgluBarriereKontakt() && fischAufgenommen) {            
            stop();
            setRotation(getRotation() - 180); 
            Greenfoot.playSound("IgluAbprallen.wav");
            if (getX() >= 150) {
                setLocation(getX() + 20, getY());
            }
            if (getX() < 150) {
                setLocation(getX(), getY() - 20);
            }
            addForce(new Vector(getRotation(), 3.0));
        }
    }
    
    // Gibt einen Rueckgabewert, ob die Barrieren beruehrt werden.
    public boolean checkIgluBarriereKontakt() {
        Actor iglubarriere = (IgluBarriere) getOneObjectAtOffset(0, 0, IgluBarriere.class);       
        return iglubarriere != null;
    } 
    
    // Fisch kann abgegeben werden, wenn die boolsche Variable wahr ist, dass Glitschi Fisch hat
    public void fischAbgeben() {
        Level lvl = (Level) getWorld();
        if (fischAufgenommen && checkFinishKontakt()) {
            punkte++;           
            fischAufgenommen = false;            
            lvl.giveFisch();
            stop();
            setRotation(GLITSCHI_STANDARD_ROTATION + 45);
            getWorld().addObject(new Laufzeitangabe(1, 1, "+1"), getWorld().getWidth() / 2, 90);
            getWorld().addObject(new Oel(), Greenfoot.getRandomNumber(400) + 100, Greenfoot.getRandomNumber(400) + 50);
            if (punkte <= 4) {
                Greenfoot.playSound("FischAbgegeben.wav");
            }
            if (punkte == 3) {
                getWorld().addObject(new Wasser(), Greenfoot.getRandomNumber(400) + 100, Greenfoot.getRandomNumber(400) + 50);
            }
            if (punkte == 4) {
                getWorld().addObject(new Oel(), Greenfoot.getRandomNumber(400) + 100, Greenfoot.getRandomNumber(400) + 50);
            }
        }
    }
    
    // Gibt einen Rueckgabewert, ob die Trennlinie beruehrt wird.
    public boolean checkFinishKontakt() {
        Finish fine = (Finish) getOneIntersectingObject(Finish.class);       
        return fine != null;
    }  
    
    // Zeigt das GameOver-Textfeld und stoppt Greenfoot.
    public void checkTod() {
        Level level = (Level) getWorld();
        if (lives <= 0) {
            getWorld().addObject( new FadeOut(), getWorld().getWidth() / 2, getWorld().getHeight() / 2);
            getWorld().addObject( new GameOver(punkte), getWorld().getWidth() / 2, getWorld().getHeight() / 2);
            Greenfoot.playSound("GameOver.wav");
            level.gameOverBild();
        }
    }
    
    // Verliert einen Punkt, wenn Oel den Fisch beruehrt.
    public void minusPunkt() {
        if (punkte > 0) {
            this.punkte--;
            Greenfoot.playSound("OhNo.wav");
            getWorld().addObject(new Laufzeitangabe(0, 0, "-1"), getWorld().getWidth() / 2, 90);
        }
        if (punkte == 0) {
            Greenfoot.playSound("OelFisch.wav");
        }
    }
    
    // Glitschi richtig hinsetzen!
    public void setzeGlitschi() {
        if (getX() <= 10 && getY() >= 590) {
            setLocation(30, 570);
        }
    }
    
    // Gibt die Momentanen Punkte aus
    public int gibPunkte() {
        return this.punkte;
    }
       
}
